%% Preprocess UWA WIHSI Database
% Author: Zohaib Khan, University of Western Australia.
% email: zohaib@csse.uwa.edu.au
%
% Please Cite:
%
% [1] Z. Khan, F. Shafait and A. Mian, "Hyperspectral Imaging for Ink
% Mismatch Detection", in 12th International Conference on Document
% Analysis and Recognition (ICDAR), pp. 877-881, 2013.

clear
clc

load('word_rect'); load('page_rect');
root_path = 'UWA WIHSI/'; norm_path = 'UWA WIHSI/';
ink_name = {'Blue', 'Black'}; numInks = length(ink_name);

img_res = [480 752 33]; threshBand = 25;

numSamp = 7; numClass = 5;

for i = 1:numInks
    for r = 1:numSamp
        %% Read Raw Hyperspectral Image
        img_orig = uint8(zeros(img_res));
        for b = 1:img_res(3)
            img_orig(:,:,b) = imread([root_path ink_name{i} sprintf('/raw/%04d/IMG_%02d.png',r,b)]);
        end
        %% Compute Binary Mask
        img_mask = ~im2bw_sauvola_fast(img_orig(:,:,threshBand)); % Sauvola Binarization
        
        %% Illumination Normalization of Document Hyperspectral Image
        img_orig = reshape(normr(reshape(double(img_orig),[],img_res(3))),img_res);
        
        %% Segment and Save Processed Handwritten Notes
        numCol = 1; numRows = 5;
        margX = rect{i}{r}(3)/numCol*0.01; margY = rect{i}{r}(4)/numClass*0.05; % X and Y interior margins
        for x = 1:numCol
            for y = 1:numRows
                ref = img_orig(round(margX + rect{i}{r}(2)+(y-1)*rect{i}{r}(4)/numClass:rect{i}{r}(2)+y*rect{i}{r}(4)/numClass-margX),round(margY+rect{i}{r}(1)+(x-1)*rect{i}{r}(3)/numCol:rect{i}{r}(1)+x*rect{i}{r}(3)/numCol-margY),:);
                lbl = img_mask(round(margX + rect{i}{r}(2)+(y-1)*rect{i}{r}(4)/numClass:rect{i}{r}(2)+y*rect{i}{r}(4)/numClass-margX),round(margY+rect{i}{r}(1)+(x-1)*rect{i}{r}(3)/numCol:rect{i}{r}(1)+x*rect{i}{r}(3)/numCol-margY));
                wrd = rect2bw(size(lbl),rect_word{i}{(r-1)*5+y}); % Create Words Mask from Rectangle
                for b = 1:img_res(3)
                    imwrite(ref(:,:,b),[norm_path ink_name{i} sprintf('/proc/images/%04d/IMG_%02d_%02d.png',r,y,b)],'PNG');
                end
                imwrite(lbl.*wrd,[norm_path ink_name{i} sprintf('/proc/masks/%04d/IMG_%02d.png',r,y)],'PNG');
            end
        end
    end
end